--- chan_zap.c.orig	Sat Mar 26 02:30:10 2005
+++ chan_zap.c	Sat Mar 26 15:31:58 2005
@@ -1617,7 +1617,7 @@
 			
 			/* Don't send audio while on hook, until the call is answered */
 			p->dialing = 1;
-			if (p->use_callerid) {
+			if (p->use_callerid && !(p->cid_signalling == CID_SIG_DTMF && p->cid_start == CID_START_POLARITY)) {
 				/* Generate the Caller-ID spill if desired */
 				if (p->cidspill) {
 					ast_log(LOG_WARNING, "cidspill already exists??\n");
@@ -1643,6 +1643,28 @@
 				p->cidrings = p->sendcalleridafter;
 			}
 
+			if (p->cid_signalling == CID_SIG_DTMF && p->cid_start == CID_START_POLARITY) {
+				x = ZT_OFFHOOK;
+				if (ioctl(p->subs[SUB_REAL].zfd, ZT_HOOK, &x)) {
+					ast_log(LOG_WARNING, "Unable to go off-hook for DTMF CLIP signalling: %s\n", strerror(errno));
+					ast_mutex_unlock(&p->lock);
+					return -1;
+				}
+				p->dop.op = ZT_DIAL_OP_REPLACE;
+				if (ast->cid.cid_num) {
+					snprintf(p->dop.dialstr, sizeof(p->dop.dialstr), "TD%sCw", ast->cid.cid_num);
+				} else {
+					snprintf(p->dop.dialstr, sizeof(p->dop.dialstr), "TB00Cw");
+				}
+				if (ioctl(p->subs[SUB_REAL].zfd, ZT_DIAL, &p->dop)) {
+					x = ZT_ONHOOK;
+					ioctl(p->subs[SUB_REAL].zfd, ZT_HOOK, &x);
+					ast_log(LOG_WARNING, "Sending DTMF CLIP failed: %s\n", strerror(errno));
+					ast_mutex_unlock(&p->lock);
+					return -1;
+				}
+				p->dop.dialstr[0] = '\0';
+			}
 
 			/* nick@dccinc.com 4/3/03 mods to allow for deferred dialing */
 			c = strchr(dest, '/');
@@ -1659,11 +1681,13 @@
 			} else {
 				p->dop.dialstr[0] = '\0';
 			}
-			x = ZT_RING;
-			if (ioctl(p->subs[SUB_REAL].zfd, ZT_HOOK, &x) && (errno != EINPROGRESS)) {
-				ast_log(LOG_WARNING, "Unable to ring phone: %s\n", strerror(errno));
-				ast_mutex_unlock(&p->lock);
-				return -1;
+			if (!(p->cid_signalling == CID_SIG_DTMF && p->cid_start == CID_START_POLARITY)) {
+				x = ZT_RING;
+				if (ioctl(p->subs[SUB_REAL].zfd, ZT_HOOK, &x) && (errno != EINPROGRESS)) {
+					ast_log(LOG_WARNING, "Unable to ring phone: %s\n", strerror(errno));
+					ast_mutex_unlock(&p->lock);
+					return -1;
+				}
 			}
 			p->dialing = 1;
 		} else {
@@ -3271,6 +3295,10 @@
 				return NULL;
 			}
 			if (!x) { /* if not still dialing in driver */
+				if (p->sig == SIG_FXOKS || p->sig == SIG_FXOLS || p->sig == SIG_FXOGS) { // Sent DTMF CLIP -- now Ring phone
+					zt_ring_phone(p);
+					break;
+				}
 				zt_enable_ec(p);
 				if (p->echobreak) {
 					zt_train_ec(p);
